@@ -1,4 +1,9 @@
 #!/usr/bin/perl
+##########################
+# CAUTION: cannot merge until new HarfBuzz::Shaper released
+#          ditto for Content.pm changes
+# Note to maintainer: don't forget to refresh HarfBuzz_example.pdf
+##########################
 # demonstrate some usage of HarfBuzz::Shaper and related text calls
 # outputs HarfBuzz.pdf
 # author: Phil M Perry
@@ -6,8 +11,8 @@
 use strict;
 use warnings;
 
-our $VERSION = '3.018'; # VERSION
-my $LAST_UPDATE = '3.018'; # manually update whenever code is changed
+our $VERSION = '3.019'; # VERSION
+my $LAST_UPDATE = '3.019'; # manually update whenever code is changed
 
 my $PDFname = $0;
    $PDFname =~ s/\..*$//;  # remove extension such as .pl
@@ -33,7 +38,7 @@ my $doliga = 1; # built-in ligatures
 
 my $fontsize = 20;
 my $dump = 0; # 1 to dump Shaper data, 0 no
-my $showNextWrite = 0; # 1 to write | after text (at next position)
+my $showNextWrite = 1; # 1 to write | after text (at next position)
 my $strike = 'none';  # or 'auto' strikethrough
 my $under = 'none';  # or 'auto' underlining (only for alignment examples)
 my $leading = 1.6; # baseline-to-baseline * font size
@@ -195,6 +200,7 @@ my %samples = (
 
 my $page = $pdf->page();
 my $text = $page->text();
+my $grfx = $page->gfx();
 my $y = 750;
 my $hb = HarfBuzz::Shaper->new();
 my ($fontfile, $info, $startx, $starty);
@@ -288,13 +294,7 @@ foreach my $samp (sort keys %samples) {
     } # end of array of chunks for one sample
 
     # show where next write would go
-    if ($showNextWrite) {
-        if ($dir eq 'L') { 
-	    $text->text_left('|>'); 
-        } else { 
-	    $text->text_right('<|'); 
-        }
-    }
+    nextWrite($dir);
 
     $y -= $leading*$fontsize;
 
@@ -311,10 +311,10 @@ $text->translate($xpos[1], 100); $text->text_center('B (begin)');
 $text->translate($xpos[2], 100); $text->text_center('C (centered)');
 $text->translate($xpos[3], 100); $text->text_center('E (end)');
 foreach my $x (@xpos) {
-    $text->translate($x,75); $text->text_center('|');
-    $text->translate($x,75); $text->text_center('|');
-    $text->translate($x,45); $text->text_center('|');
-    $text->translate($x,45); $text->text_center('|');
+    $grfx->poly($x,70, $x,90); $grfx->stroke();
+    $grfx->poly($x,40, $x,60); $grfx->stroke();
+    $grfx->circle($x,75, 2); $grfx->fill();
+    $grfx->circle($x,45, 2); $grfx->fill();
 }
 
 # first line is Latin text
@@ -334,9 +334,7 @@ for my $i (0 .. 3) {
     if ($settings{'align'} eq '') { delete $settings{'align'}; }
     $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
     # show where next write would go
-    if ($showNextWrite) {
-	$text->text_left('|>'); 
-    }
+    nextWrite('L');
 }
 
 # second line is Arabic text
@@ -357,16 +355,328 @@ for my $i (0 .. 3) {
     $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
     
     # show where next write would go
-    if ($showNextWrite) {
-    	$text->text_right('<|'); 
+    nextWrite('R');
+
+}
+
+###### second page with vertical orientation text
+$page = $pdf->page();
+$text = $page->text();
+$grfx = $page->gfx();
+$y = 750;
+#$doliga = 0; # suppress ligatures  NOT necessary?
+#$dokern = 0; # suppress kerning  NOT necessary?
+
+%samples = (
+   # Latin (English) text top to bottom
+   'TTBLatin'     => { 'title' => 'TTBLatin',
+	          'fontFile' => $latinFont,
+		  'dir' => 'T', 
+		  'script' => 'Latn',
+		  'text' => ["Hi There!"] },
+
+   # Latin (English) text bottom to top
+   # note that Shaper returns reversed text to be rendered TTB
+   'BTTLatin'     => { 'title' => 'BTTLatin',
+	          'fontFile' => $latinFont,
+		  'dir' => 'B', 
+		  'script' => 'Latn',
+		  'text' => ["Hi There!"] },
+
+   # some random Chinese characters. most interested in what direction is
+   # the default, and what is settable
+   'TTBChinese' => { 'title' => 'TTBChinese',
+   	        'fontFile' => '/Program Files (x86)/Adobe/Acrobat Reader DC/Resource/CIDFont/AdobeMingStd-Light.otf',
+ 	  	'dir' => 'T',  
+ 	  	'script' => 'Chin',
+ 	  	'text' => ["\x{5A40}\x{5A41}\x{5A42}\x{5A43}", " PDF::Builder ", "\x{5A44}\x{5A45}"] },
+
+   # Languages which are normally RTL don't seem to behave with TTB.
+   # I would expect them to be reversed, but they aren't. Maybe the direction
+   # override is scrambling HarfBuzz?
+   #
+   # Hebrew (normally RTL) and Latin (LTR) TTB direction
+   'TTBHebrew' => { 'title' => "TTBHebrew", 
+         	 'fontFile' => '/WINDOWS/Fonts/times.ttf', # has Hebrew too
+		 'dir' => 'T',
+		 'script' => 'Hebr',
+		 'text' => ["\x{05e9}\x{05dc}\x{05d5}\x{05dd} \x{05e9}\x{05de}\x{05d9}", " PDF ", "\x{05d5}\x{05d0}\x{05e0}\x{05d9}"] },
+ 
+   # A cursive script such as Arabic needs to be drawn as unconnected individual
+   # (standalone) glyphs, but still, I would expect the order to be reversed.
+   # Arabic (normally RTL) and Latin (LTR) TTB direction
+   'TTBArabic' => { 'title' => "TTBArabic", 
+         	 'fontFile' => $arabicFont,
+		 'dir' => 'T',
+		 'script' => 'Arab',
+                 'text' => ["\x{0627}\x{0644}\x{0639}\x{0631}\x{0628}\x{064a}\x{0629}", " PDF ", "\x{0627}\x{0644}\x{0639}"] },
+
+   # Hebrew (normally RTL) and Latin (normally LTR) BTT direction
+   'BTTHebrew' => { 'title' => "BTTHebrew", 
+         	 'fontFile' => '/WINDOWS/Fonts/times.ttf', # has Hebrew too
+		 'dir' => 'B',
+		 'script' => 'Hebr',
+		 'text' => ["\x{05e9}\x{05dc}\x{05d5}\x{05dd} \x{05e9}\x{05de}\x{05d9}", " NOT rec. ", "\x{05d5}\x{05d0}\x{05e0}\x{05d9}"] },
+ 
+   # Arabic (normally RTL) and Latin (normally LTR) BTT direction
+   'BTTArabic' => { 'title' => "BTTArabic", 
+         	 'fontFile' => $arabicFont,
+		 'dir' => 'B',
+		 'script' => 'Arab',
+                 'text' => ["\x{0627}\x{0644}\x{0639}\x{0631}\x{0628}\x{064a}\x{0629}", " NOT rec. ", "\x{0627}\x{0644}\x{0639}"] },
+
+);
+
+my $depth = 1; # indentation
+foreach my $samp ('TTBLatin', 'BTTLatin', 'TTBChinese', 'BTTHebrew', 'BTTArabic', 'TTBHebrew', 'TTBArabic') {  # in order given
+    $settings{'dump'} = $dump;  ### diagnostic dump to STDOUT
+    my $title = $samples{$samp}->{'title'};
+    $settings{'script'} = $samples{$samp}->{'script'}; ### Latn, Hebr, etc.
+    $settings{'features'} = (); ### required entry
+
+    $fontfile = $samples{$samp}->{'fontFile'};
+    $text->font($labelFont, $fontsize);
+    my $dir = $samples{$samp}->{'dir'}; # L(TR), R(TL), T(TB), B(TT)
+    # no underline or strikethru for now TTB, BTT
+    if ($dir eq 'T' || $dir eq 'B') { 
+       #$strike = 'none';
+	$under  = 'none';
     }
+    $text->translate(50*$depth, $y);
+    
+    $text->text($title);
+    # same line, at x+150 write raw characters
+    print "###### start of $samp output\n";
+    my $specials = $samples{$samp}->{'specials'} || 0; # default 0
+
+    # $hb->set_language( 'en_US' ); # unnecessary?
+    # $settings{'language'} = 'en'; ###
+    # language de-DE does NOT auto-ligature ss and tz
+#   if ($samples{$samp}->{'script'} eq 'Latn') {
+#       if (!$doliga || $specials==1) {  ### not needed in textHS
+#           $hb->add_features( '-liga' );  # +liga is default, -liga works
+#           push @{ $settings{'features'} }, '-liga';
+#       }
+#       if (!$dokern || $specials==1) {  ### needed in textHS
+#           # for forceLigs, turn off kerning to keep clean
+#           $hb->add_features( '-kern' );  # shut off kerning, +kern default
+#           push @{ $settings{'features'} }, '-kern';
+#       }
+#   }
+
+    $hb->reset(); # clear out previous chunk's explicit settings
+
+    if      ($samples{$samp}->{'dir'} eq 'T') {
+        $hb->set_direction('TTB');
+    } elsif ($samples{$samp}->{'dir'} eq 'B') {
+        $hb->set_direction('BTT');
+    } elsif ($samples{$samp}->{'dir'} eq 'L') {
+        $hb->set_direction('LTR');
+    } else { # R
+        $hb->set_direction('RTL');
+    }
+
+    # horizontal non-Shaper output
+    $font = $pdf->ttfont($fontfile);
+    $text->font($font, $fontsize);
+    $text->translate(120+50*$depth, $y);
+    my $textstr = join('', @{ $samples{$samp}->{'text'} });
+    $textstr =~ s/[\x{200c}\x{200d}]//g; # remove ZWNJ, ZWJ for text() output
+    $text->text($textstr, -strikethru=>$strike, -underline=>$under);
+    $y -= $leading*$fontsize;
+
+    # write Shaped chars at x=50*$depth
+    # Shaper output is TTB even if chunk is BTT
+    # we will let 'align' value default (Left aligned for LTR)
+    $startx = 50*$depth;
+    $starty = $y;
+
+    if ($samp eq 'BTTHebrew') { $starty -= 440; }
+    if ($samp eq 'BTTArabic') { $starty -= 400; }
+
+    $settings{'dir'} = $dir; ###  L(TR), R(TL), T(TB), B(TT)
+    my %opts;
+#   my $minKern = 1;
+
+    $hb->set_font($fontfile);
+    $hb->set_size($fontsize);
+
+    my $first_chunk = 1;
+    foreach (@{ $samples{$samp}->{'text'} }) { # a "chunk" of text
+	# if no ligatures, remove ZWNJ and ZWJ (ligature suppression)
+	# otherwise may get odd extra space in middle of word
+	#if (!$doliga) {
+	#    $_ =~ s/[\x{200c}\x{200d}]//g; 
+	#}
+	 
+	# TBD here could split up a long string for paragraph/page shaping,
+	#       or wait until Shaper gives final sizes.
+	# TBD here could do line full justification (modify ax per charspace
+	#       and wordspace settings). for connected/cursive scripts, don't
+	#       change interletter spacing, just interword. Probably want to 
+	#       wait for Shaper to give final sizes.
+
+        $hb->set_text($_);
+        #
+        $info = $hb->shaper(); # output is built LTR or TTB in all cases
+
+# what was used?
+#print "language found by Shaper (or set): ".($hb->get_language())."\n";
+#print "direction found by Shaper (or set): ".($hb->get_direction())."\n";
+#print "script found by Shaper (or set): ".($hb->get_script())."\n";
+#
+#if ($samples{$samp}->{'dir'} eq 'T' || $samples{$samp}->{'dir'} eq 'B') {
+#   foreach (@{$info}) {
+#     if (defined $_->{'g'}) { print "g $_->{'g'} "; }
+#     print "ax $_->{'ax'}, ay $_->{'ay'} dx $_->{'dx'} dy $_->{'dy'}\n";
+#   }
+#}
+
+	if      ($samp eq 'TTBLatin') {
+            # for single Latin text, show alignments, output text, next write
+#           my $chunkLength = $text->advancewidthHS($info, \%settings,
+#	        	      %opts, -doKern=>$dokern, -minKern=>$minKern);
+            my $chunkLength = $text->advancewidthHS($info, \%settings, %opts);
+            # B position at top (startx, starty)
+            $text->translate($startx-15, $starty-5);
+ 	    $text->text_right('B');
+            $text->translate($startx, $starty);
+            $grfx->poly($startx-10,$starty, $startx+10,$starty);
+	    $grfx->stroke();
+	    $grfx->circle($startx,$starty, 2); 
+	    $grfx->fill();
+	    # output text
+	    $settings{'align'} = 'B';
+	    $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
+	    # show next write
+	    nextWrite($dir);
+
+            # C position lower
+	    $starty -= 1.5*$chunkLength + 50;
+            $text->translate($startx-15, $starty-5);
+ 	    $text->text_right('C');
+            $text->translate($startx, $starty);
+            $grfx->poly($startx-10,$starty, $startx+10,$starty);
+	    $grfx->stroke();
+	    $grfx->circle($startx,$starty, 2); 
+	    $grfx->fill();
+	    # output text
+	    $settings{'align'} = 'C';
+	    $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
+	    # show next write
+	    nextWrite($dir);
+
+            # E position at bottom
+	    $starty -= 1.5*$chunkLength + 50;
+            $text->translate($startx-15, $starty-5);
+ 	    $text->text_right('E');
+            $text->translate($startx, $starty);
+            $grfx->poly($startx-10,$starty, $startx+10,$starty);
+	    $grfx->stroke();
+	    $grfx->circle($startx,$starty, 2); 
+	    $grfx->fill();
+	    # output text
+	    $settings{'align'} = 'E';
+	    $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
+
+	} elsif ($samp eq 'BTTLatin') {
+            # for single Latin text, show alignments, output text, next write
+#           my $chunkLength = $text->advancewidthHS($info, \%settings,
+#	        	      %opts, -doKern=>$dokern, -minKern=>$minKern);
+            my $chunkLength = $text->advancewidthHS($info, \%settings, %opts);
+            # B position at top (startx, starty-chunkLength)
+	    $starty -= $chunkLength;
+            $text->translate($startx-15, $starty-5);
+ 	    $text->text_right('B');
+            $text->translate($startx, $starty);
+            $grfx->poly($startx-10,$starty, $startx+10,$starty);
+	    $grfx->stroke();
+	    $grfx->circle($startx,$starty, 2); 
+	    $grfx->fill();
+	    # output text
+	    $settings{'align'} = 'B';
+	    $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
+	    # show next write
+	    nextWrite($dir);
+
+            # C position lower
+	    $starty -= 0.5*$chunkLength + 50;
+            $text->translate($startx-15, $starty-5);
+ 	    $text->text_right('C');
+            $text->translate($startx, $starty);
+            $grfx->poly($startx-10,$starty, $startx+10,$starty);
+	    $grfx->stroke();
+	    $grfx->circle($startx,$starty, 2); 
+	    $grfx->fill();
+	    # output text
+	    $settings{'align'} = 'C';
+	    $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
+	    # show next write
+	    nextWrite($dir);
+
+            # E position at bottom
+	    $starty -= 0.5*$chunkLength + 50;
+            $text->translate($startx-15, $starty-5);
+ 	    $text->text_right('E');
+            $text->translate($startx, $starty);
+            $grfx->poly($startx-10,$starty, $startx+10,$starty);
+	    $grfx->stroke();
+	    $grfx->circle($startx,$starty, 2); 
+	    $grfx->fill();
+	    # output text
+	    $settings{'align'} = 'E';
+	    $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
+
+	} else {
+	    if ($first_chunk) { # only do first time
+                # mark starting position (dir TTB, align B)
+                $grfx->poly($startx-10,$starty, $startx+10,$starty);
+                $grfx->stroke();
+                $grfx->circle($startx,$starty, 2); 
+                $grfx->fill();
+	        # output text
+                $text->translate($startx, $starty);
+	        $settings{'align'} = 'B';
+		$first_chunk = 0;
+	    }
+	    $text->textHS($info, \%settings, -strikethru=>$strike, -underline=>$under);
+	}
+    } # end of array of chunks for one sample
+    # show next write
+    nextWrite($dir);
+
+    if ($dir eq 'L' || $dir eq 'R') {
+        $y -= $leading*$fontsize;
+    }
+
+    $depth++;
 }
 
 ######
 $pdf->saveas($PDFname);
 
 ###################################################################
+sub nextWrite {
+    my $dir = shift;
 
+    my @pos = $text->textpos();
+    if ($showNextWrite) {
+        if      ($dir eq 'L') { 
+	    $grfx->poly($pos[0],$pos[1], $pos[0],$pos[1]+10, $pos[0]+5,$pos[1]+5);
+        } elsif ($dir eq 'R') { 
+	    $grfx->poly($pos[0],$pos[1], $pos[0],$pos[1]+10, $pos[0]-5,$pos[1]+5);
+        } elsif ($dir eq 'T') { 
+	    $grfx->poly($pos[0]-5,$pos[1], $pos[0]+5,$pos[1], $pos[0],$pos[1]-5);
+	} else { # B
+	    $grfx->poly($pos[0]-5,$pos[1], $pos[0]+5,$pos[1], $pos[0],$pos[1]+5);
+        }
+    }
+    $grfx->close();
+    $grfx->fill();
+    return;
+}
+
+###################################################################
 # A crude filter to look at all Unicode ligatures in a chunk and return
 # a modified chunk with ligatures replacing letter combinations it can
 # find a glyph for. Note that this may violate a number of languages'
